{% extends "base.html" %}

{% block content %}
  <div class="span12">
    <p class="lead">
      This project is intended to be used as a testing environment for
      Django Endless Pagination.
    </p>
    <p class="lead">
      This project also contains a basic collection
      of examples on how to use this application, providing both the ability
      to manually test Django Endless Pagination in the browser, and a demo
      showing some of the application features in action.
    </p>
    <p class="lead">
      The documentation is
      <a href="http://django-endless-pagination.readthedocs.org/">avaliable online</a>
      or in the docs directory of the project.<br />
      The source code for this app is hosted at
      <a href="https://github.com/frankban/django-endless-pagination">Github</a> and
      <a href="https://bitbucket.org/frankban/django-endless-pagination">Bitbucket</a>.
    </p>
    <p>
      The template for this project is realized using the
      <a href="http://twitter.github.com/bootstrap/index.html"
        target="_blank">Bootstrap framework</a>.
      However, Bootstrap is not required in order to use
      Django Endless Pagination.
    </p>
  </div>
{% endblock %}
